
; ===============================================================================
;			
;    -- EditBox_VirtualAlloc.asm --
;			Jay Sullivan
;
;
;	Description:  This program allocates memory for
;				an edit control using VirtualAlloc.
;
;				This usually will not work, and you
;				are unable to type anything into the
;				edit box.
;
;				Strangely, the MessageBox seems to
;				interfere with the process, and
;				changing the MessageBox's parent
;				window from "0" to "hWnd" can
;				actually make the edit box work
;				correctly on some machines. 
;				(Worked on my Win XP SP2, P4 2.27 GHz, 1 GB Ram)
;
;				Regardless of the MessageBox's 
;				interference, the program does not
;				work consistenly from one computer
;				to the next.  But, the fact that
;				I have evidence that it does work
;				on some computers is VERY strange,
;				and gives me reason to believe it
;				is possible to make this work once
;				some unknown bugs are understood.
;
;
;		To see the important code, 
;			look in the WM_CREATE section of WndProc.
; ===============================================================================
  

.386
.MODEL FLAT,STDCALL
OPTION CASEMAP:NONE
 			
;------------------------------------------------
;               Header
;------------------------------------------------

	;----------------------------
	;     includes
	;----------------------------
		
		include EditBox_VirtualAlloc.inc
		
	;----------------------------
	;     Prototypes
	;----------------------------
		
		WinMain PROTO hInst:HINSTANCE,hPrevInst:HINSTANCE,CmdLine:LPSTR,CmdShow:DWORD
		WndProc PROTO hWnd:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

;------------------------------------------------
;                .CODE
;------------------------------------------------

.CODE

	Start:
		invoke GetModuleHandle, NULL
		mov hInstance,eax
		invoke GetCommandLine
		mov CommandLine, eax
		invoke WinMain, hInstance, NULL, CommandLine, SW_SHOWDEFAULT
		invoke ExitProcess,eax
		

;------------------------------------------------
;	WinMain
;------------------------------------------------
WinMain PROC hInst:HINSTANCE,hPrevInst:HINSTANCE,CmdLine:LPSTR,CmdShow:DWORD
LOCAL wc:WNDCLASSEX,
	  msg:MSG,
	  hwnd:HWND


		;------------------------------------------------
		;	Main Window Register
		;------------------------------------------------		
			mov wc.cbSize, Sizeof WNDCLASSEX
			mov wc.style, CS_HREDRAW or CS_VREDRAW
			mov wc.lpfnWndProc, Offset WndProc
			mov wc.cbClsExtra,NULL
			mov wc.cbWndExtra,NULL
			mov eax,hInst
			mov wc.hInstance,eax
			mov wc.hbrBackground, COLOR_WINDOW+1
			mov wc.lpszMenuName, NULL
			mov wc.lpszClassName, Offset szClassName
			invoke LoadIcon, NULL, IDI_APPLICATION
			mov wc.hIcon,eax
			mov wc.hIconSm,0
			invoke LoadCursor, NULL, IDC_ARROW
			mov wc.hCursor,eax
			invoke RegisterClassEx, ADDR wc
			
		;------------------------------------------------
		;	Main Window Create
		;------------------------------------------------		
				
			invoke CreateWindowEx, WS_EX_CLIENTEDGE, Offset szClassName, Offset szAppName,\
				WS_OVERLAPPEDWINDOW or WS_CLIPCHILDREN,\
				CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,\
				NULL,NULL,hInst,NULL
			mov hwnd,eax			
			
		;-----------------------------
		;	Show Window
		;-----------------------------		
			
			invoke ShowWindow, hwnd, SW_SHOWNORMAL 
			invoke UpdateWindow, hwnd		
				
		;-----------------------------
		;	Message Loop
		;-----------------------------		
			
			jmp @F
			StartLoop:
			  invoke TranslateMessage,ADDR msg
			  invoke DispatchMessage, ADDR msg
			@@:
			  invoke GetMessage,ADDR msg,NULL,0,0
			  test eax, eax
			  jnz StartLoop
			
	mov eax, msg.wParam
	ret
WinMain ENDP


;------------------------------------------------
;	WndProc
;------------------------------------------------
WndProc PROC hWnd:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM

	;-----------------------------
	 .if uMsg==WM_CREATE
	;-----------------------------
		
		
		
		;================================================
		;	Warning box with parent set to 0.
		;
		;  This is the line that breaks the edit box on some machines.
		;  Most machines it will not work anyway, though.
		;
		;			...Why?
		;================================================
		;
		;	invoke MessageBox,0,Offset szWarningMBText,Offset szWarningMBCaption,MB_OK
		
		;================================================
		;	Warning box with parent set to hWnd.
		;
		;  This does not seem to affect the "workingness,"
		;   [I use this word here because I think it is
		;    a stupid word.  Bound to be found in a 
		;    psychology book. ;)  ]
		;
		;			...Why?
		;================================================
		;
			invoke MessageBox,hWnd,Offset szWarningMBText,Offset szWarningMBCaption,MB_OK
		
		
		;------------------------------------------------
		;	Allocate memory
		;------------------------------------------------
		
			invoke VirtualAlloc, 0, 4096, MEM_RESERVE or MEM_COMMIT, PAGE_READWRITE
			mov hVirtualAlloc, eax
			
		;================================================
		;
		;	Write '1234'.   Can't see it!
		;   (Can't see it even when it "works" on
		;    some machines, in case you're wondering.)
		;
		;================================================
		
			mov dword ptr [eax], '4321'
			
		;------------------------------------------------
		;	Assert that VirtualAlloc worked.
		;------------------------------------------------
			.if eax==0
				invoke MessageBox,hWnd,Offset szErrorMBText,Offset szErrorMBCaption,MB_OK
				invoke PostQuitMessage, NULL
				xor eax,eax
				ret
			.endif
			
		;------------------------------------------------
		;	hEditBox Create
		;------------------------------------------------
		
			invoke CreateWindowEx, WS_EX_STATICEDGE, Offset szCEdit, 0, WS_VISIBLE or WS_CHILD or WS_BORDER or ES_LEFT or ES_MULTILINE or ES_NOHIDESEL or ES_AUTOVSCROLL or ES_WANTRETURN or WS_VSCROLL or WS_HSCROLL or DS_LOCALEDIT,0,0,355,335,hWnd,0,hInstance,0
			mov hEditBox,eax
	
		;------------------------------------------------
		;	Link the edit box to the VirtualAllocated section.
		;------------------------------------------------
		
			invoke SendMessage,hEditBox,EM_SETHANDLE,hVirtualAlloc,0
			
		;------------------------------------------------
		;	Set the maximum text to one page. (Well, 4096 bytes.)
		;------------------------------------------------
		
			invoke SendMessage,hEditBox,EM_SETLIMITTEXT,4095,0
			
			
			
	;-----------------------------
	 .elseif uMsg==WM_DESTROY
	;-----------------------------
		
		invoke PostQuitMessage, NULL
		
	;-----------------------------
	 .elseif uMsg==WM_SIZE
	;-----------------------------
	
		movzx eax, word ptr lParam   ; << width
		movzx ecx, word ptr lParam+2 ; << height
		invoke MoveWindow,hEditBox, 0,0, eax, ecx, TRUE
		
	;-----------------------------
	 .else
	;-----------------------------
		
		invoke DefWindowProc, hWnd, uMsg, wParam, lParam
		ret
		
	;-----------------------------
	 .endif
	;-----------------------------
	
	xor eax,eax
	ret
WndProc ENDP




End Start